@@ -22,6 +22,7 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Optional;
+import java.util.OptionalDouble;
 import java.util.OptionalInt;
 import java.util.function.Function;
 import java.util.function.Supplier;
@@ -46,10 +47,13 @@ public class BaseGridData implements GridData {
     protected List<GridColumn<?>> columns = new ArrayList<>();
     protected List<SelectedCell> selectedCells = new ArrayList<>();
     protected List<SelectedCell> selectedHeaderCells = new ArrayList<>();
-    protected int headerRowCount = 1;
+    protected int headerRowCount = 0;
 
     protected BaseGridDataIndexManager indexManager = new BaseGridDataIndexManager(this);
     protected BaseGridDataSelectionsManager selectionsManager = new BaseGridDataSelectionsManager(this);
+    private int visibleWidth;
+    private int visibleHeight;
+    private int previousVisibleWidth;
 
     public BaseGridData() {
         this(true);
@@ -71,19 +75,33 @@ public class BaseGridData implements GridData {
 
     @Override
     public void appendColumn(final GridColumn<?> column) {
+        double originalWidth = getWidth();
         column.setIndex(columns.size());
         columns.add(column);
 
+        OptionalDouble optionalOriginalWidth = OptionalDouble.of(originalWidth);
+
+        if (GridColumn.ColumnWidthMode.isAuto(column)) {
+            column.setWidth(calculateInitWidth(column, optionalOriginalWidth));
+            internalRefreshWidth(true, optionalOriginalWidth);
+        }
         selectionsManager.onInsertColumn(columns.size() - 1);
     }
 
     @Override
     public void insertColumn(final int index,
                              final GridColumn<?> column) {
+        double originalWidth = getWidth();
         column.setIndex(columns.size());
         columns.add(index,
                     column);
 
+        OptionalDouble optionalOriginalWidth = OptionalDouble.of(originalWidth);
+
+        if (GridColumn.ColumnWidthMode.isAuto(column)) {
+            column.setWidth(calculateInitWidth(column, optionalOriginalWidth));
+            internalRefreshWidth(true, optionalOriginalWidth);
+        }
         selectionsManager.onInsertColumn(index);
     }
 
@@ -116,11 +134,12 @@ public class BaseGridData implements GridData {
             }
         }
 
+        internalRefreshWidth(true, OptionalDouble.empty());
+
         selectionsManager.onDeleteColumn(index);
     }
 
     void removeColumn(final GridColumn<?> column) {
-
         final IntStream indexes = IntStream.range(0, columns.size());
         final OptionalInt columnIndex = indexes.filter(i -> column == columns.get(i)).findFirst();
 
@@ -129,6 +148,8 @@ public class BaseGridData implements GridData {
         } else {
             columns.remove(column);
         }
+
+        internalRefreshWidth(true, OptionalDouble.empty());
     }
 
     @Override
@@ -277,7 +298,7 @@ public class BaseGridData implements GridData {
 
     @Override
     public void setHeaderRowCount(final int headerRowCount) {
-        if (!(headerRowCount > 0)) {
+        if (!(headerRowCount >= 0)) {
             throw new IllegalStateException("headerRowCount");
         }
         this.headerRowCount = headerRowCount;
@@ -619,4 +640,134 @@ public class BaseGridData implements GridData {
         }
         return maxRowIndex - 1;
     }
+
+    @Override
+    public boolean refreshWidth() {
+        return internalRefreshWidth(false, OptionalDouble.empty());
+    }
+
+    @Override
+    public boolean refreshWidth(double currentWidth) {
+        return internalRefreshWidth(false, OptionalDouble.of(currentWidth));
+    }
+
+    protected boolean internalRefreshWidth(boolean changedNumberOfColumn, OptionalDouble optionalCurrentWidth) {
+
+        double visibleWidth = getVisibleWidth();
+        // this happens during initialization
+        if (visibleWidth == 0) {
+            return false;
+        }
+        // refresh is not needed if it has not been added a column and visibleWidth doesn't change (except if
+        // previousVisibleWidth is 0 so it is the first refresh)
+        if (!changedNumberOfColumn && previousVisibleWidth != 0 && visibleWidth == previousVisibleWidth) {
+            return false;
+        }
+
+        GridWidthMetadata gridWidthMetadata = new GridWidthMetadata(optionalCurrentWidth);
+
+        // if there are no columns with AUTO width no need to continue
+        if (gridWidthMetadata.numberOfAutoColumn == 0) {
+            return false;
+        }
+
+        // verify if grid was 100% width with a delta
+        boolean wasFullWidth = Math.abs(gridWidthMetadata.currentGrossWidth - previousVisibleWidth) < 0.1;
+
+        // keep 100% width or max between visible area and grid width
+        double targetGrossWidth = wasFullWidth ? visibleWidth : Math.max(visibleWidth, gridWidthMetadata.currentGrossWidth);
+
+        double currentWidth = getWidth() - gridWidthMetadata.fixedWidth;
+        double targetWidth = targetGrossWidth - gridWidthMetadata.fixedWidth;
+
+        // if grid is greater than visible panel and it wasn't at full width no refresh is needed
+        if (visibleWidth < gridWidthMetadata.currentGrossWidth && !wasFullWidth) {
+            return false;
+        }
+
+        boolean toRedraw = false;
+        for (GridColumn<?> column : getColumns()) {
+            if (!column.isVisible() || !GridColumn.ColumnWidthMode.isAuto(column)) {
+                continue;
+            }
+            double oldWidth = column.getWidth();
+            double ratio = oldWidth / currentWidth;
+            double newWidth = ratio * targetWidth;
+            // this could happen during initialization when columns can be added before the first call to setVisibleSizeAndRefresh
+            if (oldWidth == 0) {
+                newWidth = calculateInitWidth(column, OptionalDouble.empty());
+            }
+            if (newWidth < column.getMinimumWidth()) {
+                newWidth = column.getMinimumWidth();
+            }
+            // if nothing changed no need to update nor refresh
+            if (newWidth != oldWidth) {
+                column.setWidth(newWidth);
+                toRedraw = true;
+            }
+        }
+        return toRedraw;
+    }
+
+    protected double getWidth() {
+        return getColumns().stream().filter(GridColumn::isVisible).mapToDouble(GridColumn::getWidth).sum();
+    }
+
+    @Override
+    public boolean setVisibleSizeAndRefresh(int width, int height) {
+        this.previousVisibleWidth = this.visibleWidth;
+        this.visibleWidth = width;
+        this.visibleHeight = height;
+        return refreshWidth();
+    }
+
+    @Override
+    public int getVisibleWidth() {
+        return this.visibleWidth;
+    }
+
+    @Override
+    public int getVisibleHeight() {
+        return this.visibleHeight;
+    }
+
+    double calculateInitWidth(GridColumn<?> column, OptionalDouble optionalCurrentWidth) {
+        if (!GridColumn.ColumnWidthMode.isAuto(column)) {
+            return column.getWidth();
+        }
+        GridWidthMetadata gridWidthMetadata = new GridWidthMetadata(optionalCurrentWidth);
+        int visibleWidth = getVisibleWidth();
+
+        if (gridWidthMetadata.numberOfAutoColumn < 2) {
+            return visibleWidth - gridWidthMetadata.fixedWidth;
+        }
+        return (visibleWidth - gridWidthMetadata.fixedWidth) / (gridWidthMetadata.numberOfAutoColumn - 1);
+    }
+
+    private class GridWidthMetadata {
+
+        // total size of the grid
+        private double currentGrossWidth = 0;
+        private double previousWidth = 0;
+        private long numberOfAutoColumn = 0;
+        // total size of FIXED column
+        private double fixedWidth = 0;
+
+        private GridWidthMetadata(OptionalDouble optionalCurrentWidth) {
+            for (GridColumn<?> column : getColumns()) {
+                if (!column.isVisible()) {
+                    continue;
+                }
+                double columnWidth = column.getWidth();
+
+                currentGrossWidth += columnWidth;
+                numberOfAutoColumn = GridColumn.ColumnWidthMode.isAuto(column) ? numberOfAutoColumn + 1 : numberOfAutoColumn;
+                fixedWidth = GridColumn.ColumnWidthMode.isFixed(column) ? fixedWidth + columnWidth : fixedWidth;
+            }
+            previousWidth = optionalCurrentWidth.orElse(currentGrossWidth);
+            if (optionalCurrentWidth.isPresent()) {
+                currentGrossWidth = optionalCurrentWidth.getAsDouble();
+            }
+        }
+    }
 }
