@@ -23,6 +23,7 @@ import com.google.gwt.event.dom.client.KeyDownEvent;
 import com.google.gwt.event.dom.client.KeyDownHandler;
 import org.uberfire.ext.wires.core.grids.client.model.GridColumn;
 import org.uberfire.ext.wires.core.grids.client.model.GridData;
+import org.uberfire.ext.wires.core.grids.client.widget.dom.HasDOMElementResources;
 import org.uberfire.ext.wires.core.grids.client.widget.dom.single.HasSingletonDOMElementResource;
 import org.uberfire.ext.wires.core.grids.client.widget.grid.GridWidget;
 import org.uberfire.ext.wires.core.grids.client.widget.grid.impl.KeyboardOperation.TriStateBoolean;
@@ -114,7 +115,9 @@ public class BaseGridWidgetKeyboardHandler implements KeyDownHandler {
         for (GridColumn<?> column : gridModel.getColumns()) {
             if (column instanceof HasSingletonDOMElementResource) {
                 ((HasSingletonDOMElementResource) column).flush();
-                ((HasSingletonDOMElementResource) column).destroyResources();
+            }
+            if (column instanceof HasDOMElementResources) {
+                ((HasDOMElementResources) column).destroyResources();
             }
         }
     }
