@@ -3,7 +3,7 @@
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
@@ -11,24 +11,31 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
-*/
+ */
 
 package org.guvnor.structure.backend.organizationalunit;
 
+import java.io.File;
+import java.net.URI;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
+import java.util.stream.Collectors;
 import javax.annotation.PostConstruct;
 import javax.enterprise.context.ApplicationScoped;
 import javax.enterprise.event.Event;
 import javax.enterprise.event.Observes;
 import javax.inject.Inject;
+import javax.inject.Named;
 
 import org.guvnor.structure.backend.backcompat.BackwardCompatibleUtil;
-import org.guvnor.structure.backend.config.OrgUnit;
+import org.guvnor.structure.backend.repositories.ConfiguredRepositories;
 import org.guvnor.structure.config.SystemRepositoryChangedEvent;
+import org.guvnor.structure.contributors.Contributor;
 import org.guvnor.structure.organizationalunit.NewOrganizationalUnitEvent;
 import org.guvnor.structure.organizationalunit.OrganizationalUnit;
 import org.guvnor.structure.organizationalunit.OrganizationalUnitService;
@@ -38,6 +45,7 @@ import org.guvnor.structure.organizationalunit.RepoRemovedFromOrganizationalUnit
 import org.guvnor.structure.organizationalunit.UpdatedOrganizationalUnitEvent;
 import org.guvnor.structure.repositories.Repository;
 import org.guvnor.structure.repositories.RepositoryEnvironmentUpdatedEvent;
+import org.guvnor.structure.repositories.RepositoryService;
 import org.guvnor.structure.server.config.ConfigGroup;
 import org.guvnor.structure.server.config.ConfigItem;
 import org.guvnor.structure.server.config.ConfigType;
@@ -45,13 +53,24 @@ import org.guvnor.structure.server.config.ConfigurationFactory;
 import org.guvnor.structure.server.config.ConfigurationService;
 import org.guvnor.structure.server.organizationalunit.OrganizationalUnitFactory;
 import org.jboss.errai.bus.server.annotations.Service;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import org.uberfire.ext.security.management.api.event.UserDeletedEvent;
+import org.uberfire.io.IOService;
+import org.uberfire.java.nio.file.FileSystemAlreadyExistsException;
+import org.uberfire.java.nio.file.Path;
+import org.uberfire.java.nio.fs.jgit.JGitPathImpl;
 import org.uberfire.rpc.SessionInfo;
 import org.uberfire.security.authz.AuthorizationManager;
+import org.uberfire.spaces.Space;
+import org.uberfire.spaces.SpacesAPI;
 
 @Service
 @ApplicationScoped
 public class OrganizationalUnitServiceImpl implements OrganizationalUnitService {
 
+    private static final Logger logger = LoggerFactory.getLogger(OrganizationalUnitServiceImpl.class);
+
     private ConfigurationService configurationService;
 
     private ConfigurationFactory configurationFactory;
@@ -76,10 +95,22 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
 
     Map<String, OrganizationalUnit> registeredOrganizationalUnits = new TreeMap<>(String.CASE_INSENSITIVE_ORDER);
 
+    private SpacesAPI spaces;
+
+    private RepositoryService repositoryService;
+
+    private IOService ioService;
+
+    private ConfiguredRepositories configuredRepositories;
+
+    public OrganizationalUnitServiceImpl() {
+    }
+
     @Inject
     public OrganizationalUnitServiceImpl(final ConfigurationService configurationService,
                                          final ConfigurationFactory configurationFactory,
                                          final OrganizationalUnitFactory organizationalUnitFactory,
+                                         final RepositoryService repositoryService,
                                          final BackwardCompatibleUtil backward,
                                          final Event<NewOrganizationalUnitEvent> newOrganizationalUnitEvent,
                                          final Event<RemoveOrganizationalUnitEvent> removeOrganizationalUnitEvent,
@@ -87,10 +118,14 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
                                          final Event<RepoRemovedFromOrganizationalUnitEvent> repoRemovedFromOrgUnitEvent,
                                          final Event<UpdatedOrganizationalUnitEvent> updatedOrganizationalUnitEvent,
                                          final AuthorizationManager authorizationManager,
-                                         final SessionInfo sessionInfo) {
+                                         final SpacesAPI spaces,
+                                         final SessionInfo sessionInfo,
+                                         @Named("ioStrategy") final IOService ioService,
+                                         final ConfiguredRepositories configuredRepositories) {
         this.configurationService = configurationService;
         this.configurationFactory = configurationFactory;
         this.organizationalUnitFactory = organizationalUnitFactory;
+        this.repositoryService = repositoryService;
         this.backward = backward;
         this.newOrganizationalUnitEvent = newOrganizationalUnitEvent;
         this.removeOrganizationalUnitEvent = removeOrganizationalUnitEvent;
@@ -98,12 +133,15 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
         this.repoRemovedFromOrgUnitEvent = repoRemovedFromOrgUnitEvent;
         this.updatedOrganizationalUnitEvent = updatedOrganizationalUnitEvent;
         this.authorizationManager = authorizationManager;
+        this.spaces = spaces;
         this.sessionInfo = sessionInfo;
+        this.ioService = ioService;
+        this.configuredRepositories = configuredRepositories;
     }
 
     @PostConstruct
     public void loadOrganizationalUnits() {
-        Collection<ConfigGroup> groups = configurationService.getConfiguration(ConfigType.ORGANIZATIONAL_UNIT);
+        Collection<ConfigGroup> groups = configurationService.getConfiguration(ConfigType.SPACE);
         if (groups != null) {
             for (ConfigGroup groupConfig : groups) {
                 // Make sure existing Organizational Units are correctly initialized with a default group id.
@@ -118,6 +156,29 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
                 OrganizationalUnit ou = organizationalUnitFactory.newOrganizationalUnit(groupConfig);
                 registeredOrganizationalUnits.put(ou.getName(),
                                                   ou);
+                createSpaceConfigRepositoryIfNecessary(ouName);
+            }
+        }
+        configuredRepositories.reloadRepositories();
+    }
+
+    public void userRemoved(final @Observes UserDeletedEvent event) {
+        final String removedUserIdentifier = event.getIdentifier();
+        for (OrganizationalUnit organizationalUnit : getAllOrganizationalUnits()) {
+            final boolean userRemoved = organizationalUnit.getContributors().removeIf(c -> c.getUsername().equals(removedUserIdentifier));
+            if (userRemoved) {
+                updateOrganizationalUnit(organizationalUnit.getName(),
+                                         organizationalUnit.getDefaultGroupId(),
+                                         organizationalUnit.getContributors());
+            }
+
+            for (Repository repository : organizationalUnit.getRepositories()) {
+                final List<Contributor> updatedRepositoryContributors = new ArrayList<>(repository.getContributors());
+                final boolean repositoryContributorRemoved = updatedRepositoryContributors.removeIf(c -> c.getUsername().equals(removedUserIdentifier));
+                if (repositoryContributorRemoved) {
+                    repositoryService.updateContributors(repository,
+                                                         updatedRepositoryContributors);
+                }
             }
         }
     }
@@ -132,6 +193,15 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
         return new ArrayList<>(registeredOrganizationalUnits.values());
     }
 
+    @Override
+    public Collection<Space> getAllUserSpaces() {
+        return registeredOrganizationalUnits
+                .values()
+                .stream()
+                .map(ou -> spaces.getSpace(ou.getName()))
+                .collect(Collectors.toList());
+    }
+
     @Override
     public Collection<OrganizationalUnit> getOrganizationalUnits() {
         ArrayList result = new ArrayList<>();
@@ -146,23 +216,19 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
 
     @Override
     public OrganizationalUnit createOrganizationalUnit(final String name,
-                                                       final String owner,
                                                        final String defaultGroupId) {
 
         return createOrganizationalUnit(name,
-                                        owner,
                                         defaultGroupId,
                                         new ArrayList<>());
     }
 
     @Override
     public OrganizationalUnit createOrganizationalUnit(final String name,
-                                                       final String owner,
                                                        final String defaultGroupId,
                                                        final Collection<Repository> repositories) {
 
         return createOrganizationalUnit(name,
-                                        owner,
                                         defaultGroupId,
                                         repositories,
                                         new ArrayList<>());
@@ -170,10 +236,9 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
 
     @Override
     public OrganizationalUnit createOrganizationalUnit(final String name,
-                                                       final String owner,
                                                        final String defaultGroupId,
                                                        final Collection<Repository> repositories,
-                                                       final Collection<String> contributors) {
+                                                       final Collection<Contributor> contributors) {
         if (registeredOrganizationalUnits.containsKey(name)) {
             return null;
         }
@@ -182,11 +247,9 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
 
         try {
             configurationService.startBatch();
-            final ConfigGroup groupConfig = configurationFactory.newConfigGroup(ConfigType.ORGANIZATIONAL_UNIT,
+            final ConfigGroup groupConfig = configurationFactory.newConfigGroup(ConfigType.SPACE,
                                                                                 name,
                                                                                 "");
-            groupConfig.addConfigItem(configurationFactory.newConfigItem("owner",
-                                                                         owner));
             String _defaultGroupId = defaultGroupId == null || defaultGroupId.trim().isEmpty() ? getSanitizedDefaultGroupId(name) : defaultGroupId;
             groupConfig.addConfigItem(configurationFactory.newConfigItem("defaultGroupId",
                                                                          _defaultGroupId));
@@ -194,13 +257,14 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
                                                                          getRepositoryAliases(repositories)));
             groupConfig.addConfigItem(configurationFactory.newConfigItem("security:groups",
                                                                          new ArrayList<String>()));
-            groupConfig.addConfigItem(configurationFactory.newConfigItem("contributors",
+            groupConfig.addConfigItem(configurationFactory.newConfigItem("space-contributors",
                                                                          contributors));
             configurationService.addConfiguration(groupConfig);
 
             newOrganizationalUnit = organizationalUnitFactory.newOrganizationalUnit(groupConfig);
             registeredOrganizationalUnits.put(newOrganizationalUnit.getName(),
                                               newOrganizationalUnit);
+            createSpaceConfigRepositoryIfNecessary(name);
 
             return newOrganizationalUnit;
         } finally {
@@ -212,8 +276,24 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
         }
     }
 
+    private void createSpaceConfigRepositoryIfNecessary(final String spaceName) {
+        final URI configPath = URI.create(SpacesAPI.resolveConfigFileSystemPath(SpacesAPI.Scheme.DEFAULT, spaceName));
+        final Map<String, Object> env = new HashMap<String, Object>() {{
+            put("init",
+                Boolean.TRUE);
+            put("internal",
+                Boolean.TRUE);
+        }};
+
+        try {
+            ioService.newFileSystem(configPath, env);
+        } catch (final FileSystemAlreadyExistsException ex) {
+            logger.info("Space " + spaceName + " config file system already exists.");
+        }
+    }
+
     private List<String> getRepositoryAliases(final Collection<Repository> repositories) {
-        final List<String> repositoryList = new ArrayList<String>();
+        final List<String> repositoryList = new ArrayList<>();
         for (Repository repo : repositories) {
             repositoryList.add(repo.getAlias());
         }
@@ -222,27 +302,22 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
 
     @Override
     public OrganizationalUnit updateOrganizationalUnit(final String name,
-                                                       final String owner,
                                                        final String defaultGroupId) {
         return updateOrganizationalUnit(name,
-                                        owner,
                                         defaultGroupId,
                                         null);
     }
 
     @Override
     public OrganizationalUnit updateOrganizationalUnit(String name,
-                                                       String owner,
                                                        String defaultGroupId,
-                                                       Collection<String> contributors) {
+                                                       Collection<Contributor> contributors) {
         final ConfigGroup thisGroupConfig = findGroupConfig(name);
 
         if (thisGroupConfig != null) {
             OrganizationalUnit updatedOrganizationalUnit = null;
             try {
                 configurationService.startBatch();
-                thisGroupConfig.setConfigItem(configurationFactory.newConfigItem("owner",
-                                                                                 owner));
                 // As per loadOrganizationalUnits(), all Organizational Units should have the default group id value set
                 String _defaultGroupId = defaultGroupId == null || defaultGroupId.trim().isEmpty() ?
                         thisGroupConfig.getConfigItemValue("defaultGroupId") : defaultGroupId;
@@ -250,7 +325,7 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
                                                                                  _defaultGroupId));
 
                 if (contributors != null) {
-                    thisGroupConfig.setConfigItem(configurationFactory.newConfigItem("contributors",
+                    thisGroupConfig.setConfigItem(configurationFactory.newConfigItem("space-contributors",
                                                                                      contributors));
                 }
 
@@ -259,6 +334,7 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
                 updatedOrganizationalUnit = organizationalUnitFactory.newOrganizationalUnit(thisGroupConfig);
                 registeredOrganizationalUnits.put(updatedOrganizationalUnit.getName(),
                                                   updatedOrganizationalUnit);
+                checkChildrenRepositoryContributors(updatedOrganizationalUnit);
 
                 return updatedOrganizationalUnit;
             } finally {
@@ -273,6 +349,16 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
         }
     }
 
+    private void checkChildrenRepositoryContributors(final OrganizationalUnit updatedOrganizationalUnit) {
+        repositoryService.getAllRepositories(updatedOrganizationalUnit.getSpace()).forEach(repository -> {
+            final List<Contributor> repositoryContributors = new ArrayList<>(repository.getContributors());
+            final boolean repositoryContributorsChanged = repositoryContributors.retainAll(updatedOrganizationalUnit.getContributors());
+            if (repositoryContributorsChanged) {
+                repositoryService.updateContributors(repository, repositoryContributors);
+            }
+        });
+    }
+
     @SuppressWarnings({"unchecked", "rawtypes"})
     @Override
     public void addRepository(final OrganizationalUnit organizationalUnit,
@@ -390,7 +476,7 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
     }
 
     protected ConfigGroup findGroupConfig(final String name) {
-        final Collection<ConfigGroup> groups = configurationService.getConfiguration(ConfigType.ORGANIZATIONAL_UNIT);
+        final Collection<ConfigGroup> groups = configurationService.getConfiguration(ConfigType.SPACE);
         if (groups != null) {
             for (ConfigGroup groupConfig : groups) {
                 if (groupConfig.getName().equals(name)) {
@@ -406,32 +492,67 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
         final ConfigGroup thisGroupConfig = findGroupConfig(groupName);
 
         if (thisGroupConfig != null) {
-            OrganizationalUnit ou = null;
+            OrganizationalUnit removedOu = null;
             try {
                 configurationService.startBatch();
+                final OrganizationalUnit originalOu = getOrganizationalUnit(groupName);
+                repositoryService.removeRepositories(originalOu.getSpace(),
+                                                     originalOu.getRepositories().stream().map(repo -> repo.getAlias()).collect(Collectors.toSet()));
                 configurationService.removeConfiguration(thisGroupConfig);
-                ou = registeredOrganizationalUnits.remove(groupName);
+                removedOu = registeredOrganizationalUnits.remove(groupName);
+                removeSpaceDirectory(removedOu.getSpace());
             } finally {
                 configurationService.endBatch();
-                if (ou != null) {
-                    removeOrganizationalUnitEvent.fire(new RemoveOrganizationalUnitEvent(ou,
+                if (removedOu != null) {
+                    removeOrganizationalUnitEvent.fire(new RemoveOrganizationalUnitEvent(removedOu,
                                                                                          getUserInfo(sessionInfo)));
                 }
             }
         }
     }
 
+    private void removeSpaceDirectory(final Space space) {
+        final URI configPathURI = URI.create(SpacesAPI.resolveConfigFileSystemPath(SpacesAPI.Scheme.DEFAULT, space.getName()));
+
+        final Path configPath = ioService.get(configPathURI);
+        final JGitPathImpl configGitPath = (JGitPathImpl) configPath;
+        final File spacePath = configGitPath.getFileSystem().getGit().getRepository().getDirectory().getParentFile().getParentFile();
+
+        ioService.delete(configPath.getFileSystem().getPath(""));
+        spacePath.delete();
+    }
+
     @Override
     public OrganizationalUnit getParentOrganizationalUnit(final Repository repository) {
-        for (OrganizationalUnit organizationalUnit : registeredOrganizationalUnits.values()) {
-            if (organizationalUnit.getRepositories() != null &&
-                    organizationalUnit.getRepositories().contains(repository)) {
-                return organizationalUnit;
+        for (final OrganizationalUnit organizationalUnit : registeredOrganizationalUnits.values()) {
+            if (organizationalUnit.getRepositories() != null) {
+                for (final Repository ouRepository : organizationalUnit.getRepositories()) {
+                    if (ouRepository.getAlias().equals(repository.getAlias())) {
+                        return organizationalUnit;
+                    }
+                }
             }
         }
         return null;
     }
 
+    @Override
+    public List<OrganizationalUnit> getOrganizationalUnits(Repository repository) {
+        final ArrayList<OrganizationalUnit> result = new ArrayList<>();
+
+        for (final OrganizationalUnit organizationalUnit : registeredOrganizationalUnits.values()) {
+            if (organizationalUnit.getRepositories() != null) {
+                for (final Repository ouRepository : organizationalUnit.getRepositories()) {
+                    if (ouRepository.getAlias().equals(repository.getAlias())) {
+                        result.add(organizationalUnit);
+                    }
+                }
+            }
+        }
+
+        return Collections.unmodifiableList(result);
+    }
+
     @Override
     public String getSanitizedDefaultGroupId(final String proposedGroupId) {
         //Only [A-Za-z0-9_\-.] are valid so strip everything else out
@@ -449,7 +570,7 @@ public class OrganizationalUnitServiceImpl implements OrganizationalUnitService
         return false;
     }
 
-    public void updateRegisteredOU(@Observes @OrgUnit SystemRepositoryChangedEvent changedEvent) {
+    public void updateRegisteredOUCache(@Observes @org.guvnor.structure.backend.config.Repository SystemRepositoryChangedEvent changedEvent) {
         registeredOrganizationalUnits.clear();
         loadOrganizationalUnits();
     }
